/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2012                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

#include <vcg/complex/complex.h>
#include <wrap/io_trimesh/import_ply.h>
#include <wrap/io_trimesh/export_ply.h>

using namespace vcg;
using namespace std;


class MyEdge;
class MyFace;
class MyVertex;
struct MyUsedTypes : public vcg::UsedTypes<	vcg::Use<MyVertex>   ::AsVertexType,
                                        vcg::Use<MyEdge>     ::AsEdgeType,
                                        vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex<MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face< MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::Color4b, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<MyUsedTypes>{};
class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};


int main()
{
  MyMesh m;
  tri::io::ImporterPLY<MyMesh>::Open(m,"2handle.ply");

  printf("Read mesh %i %i\n",m.FN(),m.VN());
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  tri::UpdateFlags<MyMesh>::FaceClearV(m);
  tri::UpdateFlags<MyMesh>::VertexClearV(m);
  int visitedCnt=0;
  std::stack<MyFace*> faceStack;
  faceStack.push(&(m.face[0]));
  faceStack.top()->SetV();
  while(!faceStack.empty())
  {
    MyFace *fp=faceStack.top();
    faceStack.pop();
    for(int i=0;i<3;i++)
    {
      if(!fp->FFp(i)->IsV())
      {
        MyFace *ffp = fp->FFp(i);
        bool badFace=false;
        int visitedVertexCnt=0;
        for(int j=0;j<3;j++)
        {
          if(ffp->FFp(j)->IsV())
            if(ffp->FFp(j) != fp) badFace =true;
          if(ffp->V(j)->IsV()) visitedVertexCnt++;
        }
        if(badFace==false && visitedVertexCnt<=2)
        {
          faceStack.push(ffp);
          ffp->SetV();
          for(int j=0;j<3;j++)
            ffp->V(j)->SetV();

          ffp->C()=Color4b::Red;
        }
      }
    }
    visitedCnt++;
  }

  printf("Visited %i faces\n",visitedCnt);
  tri::io::ExporterPLY<MyMesh>::Save(m,"out.ply",tri::io::Mask::IOM_FACECOLOR);

  return 0;
}
